var run = require('comandante')
var _ = require('lodash')
var Q = require('kew')
var ipfs = require('ipfs-api')
var waterfall = require('promise-waterfall')
var freeport = require('freeport')
var shutdown = require('shutdown-handler')
var rimraf = require('rimraf')

function configureNode (node, conf, cb) {
  waterfall(_.map(conf, function (value, key) {
    return function () {
      var def = Q.defer()
      run('ipfs', ['config', key, value])
        .on('error', function (err) { cb(err) })
        .on('end', function () { def.resolve() })
      return def.promise
    }
  })).then(function () {
    cb(null, true)
  })
}

function tempDir () {
  return dir = '/tmp/ipfs_' + (Math.random() + '').substr(2)
}

var Node = function (path, opts, disposable) {
  var env = process.env
  env.IPFS_PATH = path
  return {
    clean: true,
    pid: null,
    path: path,
    opts: opts,
    env: env,
    init: function (cb) {
      var t = this
      if (!cb) cb = opts
      var buf = ''
      run('ipfs', ['init'], {env: t.env})
        .on('error', function (err) { cb(err) })
        .on('data', function (data) { buf += data })
        .on('end', function () {
          configureNode(t, t.opts, function (err) {
            if (err) return cb(err)
            t.clean = false
            cb(null, t)
          })
        })
      shutdown.on('exit', t.shutdown.bind(t))
    },
    shutdown: function (e) {
      var t = this

      if (!t.clean && disposable) {
        e.preventDefault()
        rimraf(t.path, function (err) {
          if (err) throw err
          process.exit(0)
        })
      } else if (t.pid) {
        e.preventDefault()
        run('kill', [t.pid], {env: t.env})
          .on('error', function (err) { throw err })
          .on('end', function (err) { process.exit(0) })
        t.pid = null
      }
    },
    daemon: function (cb) {
      var t = this
      t.getConf('Addresses.API', function (err, apiPath) {
        var running = run('ipfs', ['daemon'], {env: t.env})
          .on('error', function (err) { cb(err) })
          .on('data', function (data) {
            if ((data + '').match(/API server listening/)) {
              t.pid = running.pid
              var split = apiPath.split('/')
              var port = split[split.length - 1] || 5001
              cb(null, ipfs('127.0.0.1', port), t.pid)
            }
          })
      })
    },
    getConf: function (key, cb) {
      var t = this
      var result = ''
      run('ipfs', ['config', key], {env: t.env})
        .on('error', function (err) { cb(err) })
        .on('data', function (data) { result += data })
        .on('end', function () { cb(null, result.trim()) })
    },
    stop: function (cb) {
      var t = this
      if (this.pid) {
        run('kill', [this.pid])
          .on('error', function (err) { cb(err) })
          .on('end', function () {
            t.pid = null
            cb(null)
          })
      } else {
        // not started, no problem
        cb(null)
      }
    }
  }
}

module.exports = {
  node: function (path, opts, cb) {
    cb(null, new Node(path, opts))
  },
  disposable: function (cb) {
    freeport(function (err, port) {
      if (err) throw err
      var node = new Node(tempDir(),
                          {"Addressess.Gateway": "",
                           "Addresses.API": "/ip4/127.0.0.1/tcp/" + port},
                          true)
      node.init(function (err, newnode) {
        if (err) throw err
        node.daemon(cb)
      })
    })
  }
}
